package com.marvinelsen.chinese.transliteration

@Suppress("MagicNumber")
enum class Tone {
    FIRST, SECOND, THIRD, FORTH, FIFTH;

    companion object {
        fun fromInt(number: Int) =
            fromIntOrNull(number) ?: throw IllegalArgumentException("Number $number is not a valid tone")

        fun fromIntOrNull(number: Int) = when (number) {
            1 -> FIRST
            2 -> SECOND
            3 -> THIRD
            4 -> FORTH
            5 -> FIFTH
            else -> null
        }

        fun fromDigit(digit: Char) =
            fromDigitOrNull(digit) ?: throw IllegalArgumentException("Digit $digit is not a valid tone")

        fun fromDigitOrNull(digit: Char) = when (digit) {
            '1' -> FIRST
            '2' -> SECOND
            '3' -> THIRD
            '4' -> FORTH
            '5' -> FIFTH
            else -> null
        }

        fun fromZhuyinToneMark(zhuyinToneMark: Char) = fromZhuyinToneMarkOrNull(zhuyinToneMark)
            ?: throw IllegalArgumentException("Invalid zhuyin tone mark '$zhuyinToneMark'")

        fun fromZhuyinToneMarkOrNull(zhuyinToneMark: Char) = when (zhuyinToneMark) {
            'ˊ' -> SECOND
            'ˇ' -> THIRD
            'ˋ' -> FORTH
            '˙' -> FIFTH
            else -> null
        }
    }

    fun toInt() = when (this) {
        FIRST -> 1
        SECOND -> 2
        THIRD -> 3
        FORTH -> 4
        FIFTH -> 5
    }

    fun format(transliterationSystem: TransliterationSystem) = when (transliterationSystem) {
        TransliterationSystem.ZHUYIN -> formatToZhuyin()
        TransliterationSystem.PINYIN_WITH_TONE_NUMBERS -> formatToPinyinWithToneNumbers()
        TransliterationSystem.PINYIN_WITH_TONE_MARKS -> formatToPinyinWithToneMarks()
    }

    private fun formatToPinyinWithToneNumbers() = when (this) {
        FIRST -> "1"
        SECOND -> "2"
        THIRD -> "3"
        FORTH -> "4"
        FIFTH -> "5"
    }

    private fun formatToPinyinWithToneMarks() = when (this) {
        FIRST -> "\u0304"
        SECOND -> "\u0301"
        THIRD -> "\u030C"
        FORTH -> "\u0300"
        FIFTH -> ""
    }

    private fun formatToZhuyin() = when (this) {
        FIRST -> ""
        SECOND -> "ˊ"
        THIRD -> "ˇ"
        FORTH -> "ˋ"
        FIFTH -> "˙"
    }
}
